package com.logicbig.example;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import java.util.List;

public class GetReferenceVsFindExample {

    public static void main(String[] args) {
        EntityManagerFactory emf =
                Persistence.createEntityManagerFactory("example-unit");
        try {
            persistEntity(emf);
            updateByReference(emf);
            updateByFinding(emf);
            allEmployeesQuery(emf);
        } finally {
            emf.close();
        }
    }

    private static Employee persistEntity(EntityManagerFactory emf) {
        Employee employee = new Employee("Sara Dorsey", "Admin");
        System.out.println("-- persisting employee --");
        EntityManager em = emf.createEntityManager();
        em.getTransaction().begin();
        em.persist(employee);
        em.getTransaction().commit();
        em.close();
        System.out.println("Employee persisted: " + employee);
        return employee;
    }

    private static void updateByReference(EntityManagerFactory emf) {
        System.out.println("-- updating employee by reference --");
        EntityManager em = emf.createEntityManager();
        Employee employee = em.getReference(Employee.class, 1);
        em.getTransaction().begin();
        System.out.println("-- setting new department --");
        employee.setDepartment("IT");
        System.out.println("-- commit --");
        em.getTransaction().commit();
        em.close();
    }

    private static void updateByFinding(EntityManagerFactory emf) {
        System.out.println("-- updating employee by finding --");
        EntityManager em = emf.createEntityManager();
        Employee employee = em.find(Employee.class, 1);
        em.getTransaction().begin();
        System.out.println("-- setting new name -- ");
        employee.setName("Diana");
        System.out.println("-- committing --");
        em.getTransaction().commit();
        em.close();
    }

    private static void allEmployeesQuery(EntityManagerFactory emf) {
        System.out.println("-- select all query --");
        EntityManager em = emf.createEntityManager();
        Query query = em.createQuery("select t from Employee t");
        List resultList = query.getResultList();
        resultList.forEach(System.out::println);
    }
}